Jack.Auth
=========

JSGI authorization handlers for Jack and Narwhal.


Status
------

* Basic: Completed
* Digest: Completed.
* OpenID: Outstanding


History
-------

* 2009-09-10 V0.5 Merged with Jack. Tidy up names, params, exports. These changes will break existing code.
* 2009-09-09 V0.4 Digest Authentication.
* 2009-09-04 V0.3 Conform to Jack response specification change from array to object.  BasicMiddleware() API change.
* 2009-08-26 V0.2 Changed authenticator() parameters.
* 2009-08-20 V0.1 First Release. Basic Authentication. 


Usage: Basic Authentication
---------------------------

    var BasicAuth = require("jack/auth/basic/handler").Middleware;

    var myApp = function(env) {
        return {
            status: 200,
            headers: {"Content-Type": "text/plain"},
            body: ["Hello Admin!"]
        }
    }

    // The JSGI handler is generated by the call to Middleware(app, params)
    // The params are:
    //      realm: a string declaring the authentication realm
    //      isValid(request): a function which takes an object which exposes username and password and returns
    //      true or false if the pair is accepted or rejected

    exports.app = BasicAuth(myApp, {
        realm: "my realm",
        isValid: function(request) {
            if (request.username == 'admin' && request.password == 'pass')
                return true; //allow
            return false; //deny
        }
    });


Usage: Digest Authentication
----------------------------

    var DigestAuth = require("jack/auth/digest/handler").Middleware;

    var myApp = function(env) {
        return {
            status: 200,
            headers: {"Content-Type": "text/plain"},
            body: ["Hello Admin!"]
        }
    }

    // The JSGI handler is generated by the call to Middleware(app, params)
    // The params are:
    //      realm: a string declaring the authentication realm
    //      opaque: a secret hashed and passed to the client
    //      getPassword(username): returns the password for a username

    exports.app = DigestAuth(myApp, {
        realm: "my realm",
        opaque: "this-is-a-secret",
        getPassword: function(username) {
            return {'admin': 'password'}[username];
        }
    });


Contributors
------------

* [Neville Burnell][2]


Acknowledgments
---------------

This software was inspired by [Rack::Auth][1]

[1]:http://github.com/rack/rack
[2]:http://github.com/nevilleburnell


License
-------

Copyright (c) 2009 Neville Burnell

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

